#pragma once

#include "api.hpp"
#include <framework/game.hpp>
#include <vector>
#include <string_view>
#include <memory>

namespace particles
{
    
class ParticleSimulation;

// extern "C" disables name mangling
extern "C" PARTICLES_API extern ParticleSimulation* create_simulation() noexcept;
extern "C" PARTICLES_API extern void destroy_simulation(ParticleSimulation*) noexcept;

class PARTICLES_API ParticleSimulation
{
    friend PARTICLES_API ParticleSimulation* create_simulation() noexcept;
    friend PARTICLES_API void destroy_simulation(ParticleSimulation*) noexcept;
public:
    virtual std::vector<std::string_view> const& get_particle_types() const noexcept = 0;
    virtual void spawn_particle_system(std::string_view type, fw::f32 x, fw::f32 y) = 0;
    virtual void reset() noexcept = 0;

    virtual void update(fw::Game::TimeSpan delta_time) = 0;
    virtual void render(fw::Surface& screen) noexcept = 0;

protected:
    ParticleSimulation() noexcept = default;
    virtual ~ParticleSimulation() noexcept = default;
};

struct ParticleSimulationDestroyer
{
    void operator()(ParticleSimulation* ptr) const noexcept { destroy_simulation(ptr); }
};
using ParticleSimulationPtr = std::unique_ptr<ParticleSimulation, ParticleSimulationDestroyer>;

} // namespace particles
