#pragma once

#include "particle_random.hpp"
#include <array>
#include <cmath>

/*
* Input properties.
* Simple structures used to describe how fields are initialized.
* 
* Constant and Random are both generating 1 coordinate.
* Polar generates 2 coordinates.
*/
namespace particles
{

constexpr float pi = 3.1415926535897932384626433832795f;

/*
* A constant value.
*/
struct Constant
{
    constexpr Constant() noexcept = default;
    float const value{};

    auto generate(RandomEngine&)
        const noexcept -> float
    {
        return value;
    }
};

/*
* A random value in range.
*/
struct Random
{
    constexpr Random() noexcept = default;
    float const min{};
    float const max{};

    auto generate(RandomEngine& random)
        const noexcept -> float
    {
        return std::uniform_real_distribution<float>
        { min, max }(random);
    }
};

/*
* A random polar value.
*/
struct Polar
{
    constexpr Polar() noexcept = default;
    float const range_min{};
    float const range_max{};
    float const angle_min{ 0.f };
    float const angle_max{ 2.f * pi };

    auto generate(RandomEngine& random)
        const noexcept->std::array<float, 2>
    {
        auto const length = std::uniform_real_distribution<float>{ range_min, range_max }(random);
        auto const angle = std::uniform_real_distribution<float>{ angle_min, angle_max }(random);
        return {
            std::cos(angle) * length,
            std::sin(angle) * length
        };
    }
};

} // namespace particles
