#pragma once
#include <type_traits>

namespace fw
{
    
template <typename T>
constexpr bool IsEnumBitmask = false;

template <typename In, typename Out = void>
using IfEnumBitmask = std::enable_if_t<std::is_enum_v<In> && IsEnumBitmask<In>, Out>;

template <typename T> IfEnumBitmask<T, T> operator&(T lhs, T rhs) { return static_cast<T>(static_cast<std::underlying_type_t<T>>(lhs) & static_cast<std::underlying_type_t<T>>(rhs)); }
template <typename T> IfEnumBitmask<T, T> operator|(T lhs, T rhs) { return static_cast<T>(static_cast<std::underlying_type_t<T>>(lhs) | static_cast<std::underlying_type_t<T>>(rhs)); }
template <typename T> IfEnumBitmask<T, T> operator^(T lhs, T rhs) { return static_cast<T>(static_cast<std::underlying_type_t<T>>(lhs) ^ static_cast<std::underlying_type_t<T>>(rhs)); }

template <typename T> IfEnumBitmask<T, T&> operator&=(T& lhs, T rhs) { return lhs = static_cast<T>(static_cast<std::underlying_type_t<T>>(lhs) & static_cast<std::underlying_type_t<T>>(rhs)); }
template <typename T> IfEnumBitmask<T, T&> operator|=(T& lhs, T rhs) { return lhs = static_cast<T>(static_cast<std::underlying_type_t<T>>(lhs) | static_cast<std::underlying_type_t<T>>(rhs)); }
template <typename T> IfEnumBitmask<T, T&> operator^=(T& lhs, T rhs) { return lhs = static_cast<T>(static_cast<std::underlying_type_t<T>>(lhs) ^ static_cast<std::underlying_type_t<T>>(rhs)); }

template <typename T> IfEnumBitmask<T, T> operator!(T value) { return static_cast<T>(!static_cast<std::underlying_type_t<T>>(value)); }
template <typename T> IfEnumBitmask<T, T> operator~(T value) { return static_cast<T>(~static_cast<std::underlying_type_t<T>>(value)); }

} // namespace fw
