#pragma once

#include <chrono>
#include <memory>
#include "types.hpp"
#include "io_enums.hpp"

namespace fw
{
class Surface;

class Game
{
public:
    using TimeSpan = std::chrono::duration<float, std::chrono::seconds::period>;
    using TimePoint = std::chrono::time_point<std::chrono::high_resolution_clock, TimeSpan>;

    virtual void update(Surface& screen, TimeSpan delta_time) = 0;

    virtual void on_mouse_down([[maybe_unused]] MouseButton button, [[maybe_unused]] Modifiers modifiers) { }
    virtual void on_mouse_up  ([[maybe_unused]] MouseButton button, [[maybe_unused]] Modifiers modifiers) { }

    virtual void on_mouse_move([[maybe_unused]] i32 mx, [[maybe_unused]] i32 my) { }
    virtual void on_text([[maybe_unused]] u32 character) { }
    virtual void on_key_down([[maybe_unused]] Key key, [[maybe_unused]] Modifiers modifiers, [[maybe_unused]] bool repeat) { }
    virtual void on_key_up  ([[maybe_unused]] Key key, [[maybe_unused]] Modifiers modifiers) { }

    virtual ~Game() noexcept;

    bool vsync() const noexcept;
    void vsync(bool on) noexcept;
    void resize(i32 width, i32 height, i32 scale_power = 0);
    void exit() noexcept;

    static void run(Game& game);
    static void run(Game&& game) { run(game); }

protected:
    Game(char const* title, i32 width, i32 height, i32 scale_power = 0);

private:
    class Impl;
public:
    std::unique_ptr<Impl> _;
};

} // namespace fw
