#include "shader.hpp"
#include <memory>
#include <string>

namespace fw
{

GlShader::GlShader(): shader_id_(0)
{
}

GlShader::GlShader(ShaderType type, std::string_view content)
{
    shader_id_ = glCreateShader(static_cast<GLenum>(type));
    if (shader_id_ == 0)
    {
        throw std::runtime_error{"error creating shader program"};
    }

    // Compile shader
    auto str_data = content.data();
    GLint str_length = static_cast<GLint>(content.length());
    glShaderSource(shader_id_, 1, &str_data, &str_length);
    glCompileShader(shader_id_);

    // Check results
    GLint result = GL_FALSE;
    glGetShaderiv(shader_id_, GL_COMPILE_STATUS, &result);
    GLint info_log_length;
    glGetShaderiv(shader_id_, GL_INFO_LOG_LENGTH, &info_log_length);
    if (result == GL_TRUE || info_log_length <= 0) { return; }
    
    std::string error_msg;
    error_msg.resize(info_log_length);
    glGetShaderInfoLog(shader_id_, info_log_length, nullptr, error_msg.data());
    throw std::runtime_error{std::string{"error compiling shader: "} + error_msg};
}

GlShader::GlShader(GlShader&& rhs) noexcept:
    GlShader{}
{
    swap(rhs);
}

GlShader& GlShader::operator=(GlShader&& rhs) noexcept
{
    auto move = std::move(rhs);
    swap(move);
    return *this;
}

GlShader::~GlShader()
{
    if (shader_id_ != 0)
    {
        glDeleteShader(shader_id_);
    }
}

} // namespace fw
