#pragma once
#include "render_incl.hpp"
#include <string_view>
#include <utility>

namespace fw
{
    
enum class ShaderType
{
    Vertex   = GL_VERTEX_SHADER,
    Fragment = GL_FRAGMENT_SHADER
};

class GlShaderProgram;
class GlShader
{
    friend class GlShaderProgram;
public:
    GlShader();
    explicit GlShader(ShaderType type, std::string_view content);

    GlShader(GlShader const&) = delete;
    GlShader& operator=(GlShader const&) = delete;

    GlShader(GlShader&& rhs) noexcept;
    GlShader& operator=(GlShader&& rhs) noexcept;

    ~GlShader();

    void swap(GlShader& rhs) noexcept
    {
        std::swap(shader_id_, rhs.shader_id_);
    }
    GLuint id() const noexcept { return shader_id_; }

private:
    GLuint shader_id_;
};
inline void swap(GlShader& lhs, GlShader& rhs) noexcept { lhs.swap(rhs); }

} // namespace fw
