#include "shader_program.hpp"

namespace fw
{

GlShaderProgram::GlShaderProgram() :
    shader_program_id_(0)
{
}

GlShaderProgram::GlShaderProgram(std::initializer_list<GlShader> shaders)
{
    shader_program_id_ = glCreateProgram();
    if (shader_program_id_ == 0)
    {
        throw std::runtime_error{"could not create shader program"};
    }

    for (auto const& shader : shaders)
    {
        glAttachShader(shader_program_id_, shader.shader_id_);
    }
    glLinkProgram(shader_program_id_);

    // Check results
    GLint result = GL_FALSE;
    glGetProgramiv(shader_program_id_, GL_LINK_STATUS, &result);
    GLint info_log_length;
    glGetProgramiv(shader_program_id_, GL_INFO_LOG_LENGTH, &info_log_length);
    if (result != GL_TRUE && info_log_length > 0)
    {
        std::string error_msg;
        error_msg.resize(info_log_length);
        glGetShaderInfoLog(shader_program_id_, info_log_length, nullptr, error_msg.data());
        throw std::runtime_error{std::string{"error linking shader: "} + error_msg};
    }

    for (auto const& shader : shaders)
    {
        glDetachShader(shader_program_id_, shader.shader_id_);
    }
}

GlShaderProgram::GlShaderProgram(GlShaderProgram&& rhs) noexcept :
    GlShaderProgram{}
{
    swap(rhs);
}

GlShaderProgram& GlShaderProgram::operator=(GlShaderProgram&& rhs) noexcept
{
    auto move = std::move(rhs);
    swap(move);
    return *this;
}

GlShaderProgram::~GlShaderProgram()
{
    if (shader_program_id_ != 0)
    {
        glDeleteProgram(shader_program_id_);
    }
}

} // namespace fw
