#pragma once
#include "render_incl.hpp"
#include <utility>

namespace fw
{

enum class TextureWrap
{
    ClampToEdge    = GL_CLAMP_TO_EDGE,
    ClampToBorder  = GL_CLAMP_TO_BORDER,
    MirroredRepeat = GL_MIRRORED_REPEAT,
    Repeat         = GL_REPEAT
};

enum class TextureFilter
{
    Nearest              = GL_NEAREST,
    Linear               = GL_LINEAR,
    NearestMipmapNearest = GL_NEAREST_MIPMAP_NEAREST,
    LinearMipmapNearest  = GL_LINEAR_MIPMAP_NEAREST,
    NearestMipmapLinear  = GL_NEAREST_MIPMAP_LINEAR,
    LinearMipmapLinear   = GL_LINEAR_MIPMAP_LINEAR
};
    
class Surface;
class GlTexture
{
public:
    GlTexture() noexcept;
    GlTexture(TextureWrap wrap_u, TextureWrap wrap_v, TextureFilter filter_min, TextureFilter filter_mag);

    GlTexture(GlTexture const&) = delete;
    GlTexture& operator=(GlTexture const&) = delete;

    GlTexture(GlTexture&&) noexcept;
    GlTexture& operator=(GlTexture&&) noexcept;

    ~GlTexture() noexcept;

    void update(Surface const& surface);

    void swap(GlTexture& rhs) noexcept
    {
        std::swap(tex_id_, rhs.tex_id_);
    }
    GLuint id() const noexcept { return tex_id_; }

private:
    GLuint tex_id_;
};
inline void swap(GlTexture& lhs, GlTexture& rhs) noexcept { lhs.swap(rhs); }

} // namespace fw
