#include <type_traits>
#include <random>

template <typename T, typename REng>
auto random_number(REng&& engine, T min, T max)
    -> std::enable_if_t<std::is_floating_point_v<T>, T>
{
    return std::uniform_real_distribution<T>{ min, max }(engine);
}

template <typename T, typename REng>
auto random_number(REng&& engine, T min, T max)
    -> std::enable_if_t<std::is_integral_v<T>, T>
{
    return std::uniform_int_distribution<T>{ min, max }(engine);
}

static void example()
{
    std::default_random_engine engine{ 0 };

    auto rnd1 = random_number(engine, 0.f, 100.f);
    auto rnd2 = random_number(engine, 1, 6);
    auto rnd3 = random_number<float>(engine, 0.5, 1.5);
    auto rnd4 = random_number<double>(engine, 1, 6);
    auto rnd5 = random_number<std::uint32_t>(engine, 42, 420);

    static_assert(std::is_same_v<decltype(rnd1), float        >);
    static_assert(std::is_same_v<decltype(rnd2), int          >);
    static_assert(std::is_same_v<decltype(rnd3), float        >);
    static_assert(std::is_same_v<decltype(rnd4), double       >);
    static_assert(std::is_same_v<decltype(rnd5), std::uint32_t>);
}
