// See exercise_particles/src/private/type_list.hpp

#include <type_traits>

template <typename... Types>
struct TypeList {};

//template <typename, typename>
//struct ConcatTypeListImpl;
//template <typename... TA, typename... TB>
//struct ConcatTypeListImpl<TypeList<TA...>, TypeList<TB...>>
//{
//    using Type = TypeList<TA..., TB...>;
//};
//
//template <typename TA, typename TB>
//using ConcatTypeList = typename ConcatTypeListImpl<TA, TB>::Type;

template <typename... TA, typename... TB>
constexpr auto concat_type_list(TypeList<TA...>, TypeList<TB...>)
    -> TypeList<TA..., TB...> { return {}; }

template <typename TA, typename TB>
using ConcatTypeList = decltype(concat_type_list(TA{}, TB{}));

static_assert(std::is_same_v<
    TypeList<bool, float, int, char, unsigned>,
    ConcatTypeList<TypeList<bool, float, int>, TypeList<char, unsigned>>
>);


template <size_t Index, typename First, typename... Rest>
struct IndexTypesImpl
{
    static_assert(Index <= sizeof...(Rest), "index out of range");
    using Type = typename IndexTypesImpl<
        Index - 1,
        Rest...
    >::Type;
};
template <typename First, typename... Rest>
struct IndexTypesImpl<0, First, Rest...>
{
    using Type = First;
};

template <size_t Index, typename... Types>
using IndexTypes = typename IndexTypesImpl<Index, Types...>::Type;

static_assert(std::is_same_v<
    bool,
    IndexTypes<0, bool, int, float>
>);
static_assert(std::is_same_v<
    int,
    IndexTypes<1, bool, int, float>
>);
static_assert(std::is_same_v<
    float,
    IndexTypes<2, bool, int, float>
>);


template <typename T, typename... Types>
constexpr bool ContainsType = false;
template <typename T, typename... Rest>
constexpr bool ContainsType<T, T, Rest...> = true;
template <typename T, typename J, typename... Rest>
constexpr bool ContainsType<T, J, Rest...> = ContainsType<T, Rest...>;

static_assert( ContainsType<int,   float, double, int, void>);
static_assert(!ContainsType<short, float, double, int, void>);

template <typename T, typename TypeList>
constexpr bool ListContainsType = false;
template <typename T, typename... Types>
constexpr bool ListContainsType<T, TypeList<Types...>> = ContainsType<T, Types...>;

static_assert( ListContainsType<int,   TypeList<float, double, int, void>>);
static_assert(!ListContainsType<short, TypeList<float, double, int, void>>);

template <typename T, typename... Types>
using AppendTypeUnique = std::conditional_t<
    ContainsType<T, Types...>,
    /*?*/TypeList<Types...>,
    /*:*/TypeList<Types..., T>>;

static_assert(std::is_same_v<
    TypeList<bool, int, float, double>,
    AppendTypeUnique<double, bool, int, float>
>);
static_assert(std::is_same_v<
    TypeList<bool, int, float>,
    AppendTypeUnique<int, bool, int, float>
>);

template <typename... Results>
constexpr auto unique_type_list(TypeList<Results...>, TypeList<>)
    -> TypeList<Results...> { return {}; }
template <typename... Results, typename First, typename... Inputs>
constexpr auto unique_type_list(TypeList<Results...>, TypeList<First, Inputs...>)
    -> decltype(unique_type_list(AppendTypeUnique<First, Results...>{}, TypeList<Inputs...>{}))
{ return {}; }

template <typename... Types>
using UniqueTypeList = decltype(unique_type_list(TypeList<>{}, TypeList<Types...>{}));

static_assert(std::is_same_v<
    TypeList<int, bool, double, float>,
    UniqueTypeList<int, bool, bool, double, int, float, float, double, bool>
>);
