var CodeSample;
(function (CodeSample) {
    const CHARACTER_WIDTH = 0.97948;
    class CharacterVisibilityFormatter {
        get_char_range(line, range) {
            // Find existing range
            for (const cr of line.char_visibility) {
                if (cr.range[0] === range[0] && cr.range[1] === range[1]) {
                    return cr;
                }
                assert(range[0] >= cr.range[1] || range[1] < cr.range[0], 'overlapping character hiding ranges');
            }
            // Create range
            const cr = {
                range: range,
                toggle_at: [],
                visible: true
            };
            line.char_visibility.push(cr);
            return cr;
        }
        preprocess_line(line) {
            line.char_visibility = [];
        }
        process_set(lines, set) {
            set.forEach((keyframe, keyframe_index) => {
                for (const action of keyframe) {
                    if (action.type !== 'show_characters' && action.type !== 'hide_characters') {
                        continue;
                    }
                    assert(action.range[0] >= 0 && action.range[1] <= lines[action.line_index].content.length, 'invalid character range');
                    const show = action.type === 'show_characters';
                    const cr = this.get_char_range(lines[action.line_index], action.range);
                    if (show === cr.visible) {
                        continue;
                    }
                    cr.visible = show;
                    // Prevents multiple toggles in a single keyframe
                    if (cr.toggle_at[cr.toggle_at.length - 1] === keyframe_index) {
                        cr.toggle_at.pop();
                    }
                    else {
                        cr.toggle_at.push(keyframe_index);
                    }
                }
            });
        }
        static break_elem(elem, index) {
            const text = elem.text();
            elem.text(text.substr(0, index));
            const clone = elem.clone();
            clone.text(text.substr(index));
            clone.insertAfter(elem);
            return [elem, clone];
        }
        render(lines, set) {
            const toggles = set.map(() => []);
            lines.forEach(line => {
                if (line.char_visibility.length === 0) {
                    return;
                }
                line.char_visibility.sort((a, b) => a.range[0] - b.range[0]);
                const source_line = line.source_code[0].children('.content').first();
                for (const cr of line.char_visibility) {
                    if (cr.toggle_at.length === 0) {
                        continue;
                    }
                    // Break the spans and build a list of spans that are contained
                    // in the collapsable range.
                    let contained_elements = [];
                    let index = 0;
                    let start_found = false;
                    for (let child of map(source_line.children('span:not(.marker)'), $)) {
                        let text = child.text();
                        let start = index, end = index + text.length;
                        index = end;
                        if (!start_found) {
                            if (cr.range[0] >= end) {
                                continue;
                            }
                            if (cr.range[0] !== start) {
                                const res = CharacterVisibilityFormatter.break_elem(child, cr.range[0] - start);
                                child = res[1];
                                text = text.substr(cr.range[0] - start);
                                start = cr.range[0];
                            }
                            start_found = true;
                        }
                        if (cr.range[1] === start) {
                            break;
                        }
                        contained_elements.push(child);
                        if (cr.range[1] >= end) {
                            continue;
                        }
                        CharacterVisibilityFormatter.break_elem(child, cr.range[1] - start);
                        break;
                    }
                    for (const contained_element of contained_elements) {
                        contained_element.css('width', (contained_element.text().length * CHARACTER_WIDTH) + '%');
                        contained_element.addClass('collapsable');
                    }
                    for (const toggle_index of cr.toggle_at) {
                        toggles[toggle_index].push(...contained_elements);
                    }
                }
            });
            return toggles;
        }
    }
    CodeSample.Code.formatters.push(new CharacterVisibilityFormatter());
})(CodeSample || (CodeSample = {}));
//# sourceMappingURL=character_visibility.js.map