var CodeSample;
(function (CodeSample) {
    class EllipsisFormatter {
        preprocess_line(line) { }
        process_set(lines, set) {
            // Step 1. Build an array of collapsed ranges per keyframe
            const collapsed_ranges = set.map((keyframe, keyframe_index) => {
                const res = lines.map(x => x.visibility[keyframe_index])
                    .concat([false])
                    .reduce((state, visible, i) => {
                    if (visible != state.previous_visible) {
                        if (visible) {
                            state.ranges.push([state.range_start, i - 1]);
                        }
                        else {
                            state.range_start = i;
                        }
                    }
                    state.previous_visible = visible;
                    return state;
                }, {
                    previous_visible: true,
                    range_start: 0,
                    ranges: []
                }).ranges;
                if (res.length !== 0 && res[0][0] === 0) {
                    res.splice(0, 1);
                }
                return res;
            });
            const ellipsis_lines = [];
            while (true) {
                // Step 2. Find line that is in the most ranges of ellipses.
                const histogram = new Array(lines.length).fill(0);
                for (const keyframe of collapsed_ranges) {
                    for (const range of keyframe) {
                        if (range instanceof Array) {
                            for (let i = range[0]; i <= range[1]; i++) {
                                histogram[i]++;
                            }
                        }
                    }
                }
                let max_overlap_count = Math.max(...histogram);
                if (max_overlap_count === 0) {
                    break;
                }
                let max_overlap_line = histogram.lastIndexOf(max_overlap_count);
                const output_index = ellipsis_lines.length;
                ellipsis_lines.push(max_overlap_line);
                // Step 3. Replace the ranges by the index in the output array
                let all_processed = true;
                for (const keyframe of collapsed_ranges) {
                    for (let i = 0; i < keyframe.length; i++) {
                        const range = keyframe[i];
                        if (range instanceof Array
                            && range[0] <= max_overlap_line
                            && range[1] >= max_overlap_line) {
                            keyframe[i] = output_index;
                        }
                        else if (range instanceof Array) {
                            all_processed = false;
                        }
                    }
                }
                if (all_processed) {
                    break;
                }
            }
            const ellipsis_character = String.fromCharCode(0x22EE);
            for (const ellipsis_line of ellipsis_lines) {
                lines[ellipsis_line].ellipsis = {
                    line_number: $('<span class="collapse ellipsis">').text(ellipsis_character),
                    source_code: $('<div class="collapse">'),
                    toggles: []
                };
            }
            let visible_ellipsis = [];
            collapsed_ranges.forEach((keyframe, keyframe_index) => {
                const keep_visible = [];
                for (const index of keyframe) {
                    const line = lines[ellipsis_lines[index]];
                    keep_visible.push(line.ellipsis);
                }
                for (const toggle of visible_ellipsis.filter(x => keep_visible.indexOf(x) === -1)
                    .concat(keep_visible.filter(x => visible_ellipsis.indexOf(x) === -1))) {
                    toggle.toggles.push(keyframe_index);
                }
                visible_ellipsis = keep_visible;
            });
        }
        render(lines, set) {
            for (const line of lines) {
                if (!line.ellipsis) {
                    continue;
                }
                line.line_number.push(line.ellipsis.line_number);
                line.source_code.push(line.ellipsis.source_code);
            }
            return set.map((keyframe, keyframe_index) => {
                const toggles = [];
                for (const line of lines) {
                    if (!line.ellipsis) {
                        continue;
                    }
                    for (const toggle of line.ellipsis.toggles) {
                        if (toggle === keyframe_index) {
                            toggles.push(line.ellipsis.line_number, line.ellipsis.source_code);
                        }
                    }
                }
                return toggles;
            });
        }
    }
    CodeSample.Code.formatters.push(new EllipsisFormatter());
})(CodeSample || (CodeSample = {}));
//# sourceMappingURL=ellipsis.js.map