var CodeSample;
(function (CodeSample) {
    class LineVisibilityFormatter {
        preprocess_line(line) {
            line.visibility = [];
            line.ever_visible = false;
        }
        process_set(lines, set) {
            const active_visibility = lines.map(x => false);
            for (const keyframe of set) {
                for (const action of keyframe) {
                    switch (action.type) {
                        case 'show_line':
                            active_visibility[action.line_index] = true;
                            lines[action.line_index].ever_visible = true;
                            break;
                        case 'hide_line':
                            active_visibility[action.line_index] = false;
                            break;
                    }
                }
                for (let i = 0; i < active_visibility.length; i++) {
                    lines[i].visibility.push(active_visibility[i]);
                }
            }
        }
        render(lines, set) {
            return set.map((keyframe, keyframe_index) => {
                const toggles = [];
                lines.forEach(line => {
                    const previous = keyframe_index !== 0 && line.visibility[keyframe_index - 1];
                    const current = line.visibility[keyframe_index];
                    if (previous !== current) {
                        toggles.push(...line.line_number);
                        toggles.push(...line.source_code);
                    }
                });
                return toggles;
            });
        }
    }
    CodeSample.Code.formatters.push(new LineVisibilityFormatter());
})(CodeSample || (CodeSample = {}));
//# sourceMappingURL=line_visibility.js.map