var CodeSample;
(function (CodeSample) {
    class MarkersFormatter {
        preprocess_line(line) {
            line.markers = [];
        }
        process_set(lines, set) {
            let active_marker = [];
            set.forEach((keyframe, keyframe_index) => {
                for (const action of keyframe) {
                    switch (action.type) {
                        case 'add_marker':
                            {
                                const line = lines[action.line_index];
                                const markers = line.markers;
                                const index = markers.findIndex(x => x.range[0] === action.range[0] &&
                                    x.range[1] === action.range[1] &&
                                    x.marker_type === action.marker_type);
                                if (index === -1) {
                                    const elem = $('<span class="marker collapse">')
                                        .html(`${' '.repeat(action.range[0])}<span>${' '.repeat(action.range[1] - action.range[0])}</span>`)
                                        .addClass('marker-' + action.marker_type);
                                    markers.push({
                                        range: action.range,
                                        element: elem,
                                        toggle_frames: [keyframe_index],
                                        marker_type: action.marker_type
                                    });
                                    active_marker.push(markers[markers.length - 1]);
                                    break;
                                }
                                const marker = markers[index];
                                if (active_marker.indexOf(marker) !== -1) {
                                    break;
                                }
                                marker.toggle_frames.push(keyframe_index);
                                active_marker.push(marker);
                                break;
                            }
                        case 'clear_markers':
                            {
                                active_marker = active_marker.filter(marker => {
                                    if (action.marker_types.indexOf(marker.marker_type) === -1) {
                                        return true;
                                    }
                                    marker.toggle_frames.push(keyframe_index);
                                    return false;
                                });
                                break;
                            }
                    }
                }
            });
        }
        render(lines, set) {
            for (const line of lines) {
                for (const marker of line.markers) {
                    marker.element.appendTo(line.source_code[0].find('.content'));
                }
            }
            return set.map((keyframe, keyframe_index) => {
                const toggles = [];
                for (const line of lines) {
                    for (const markers of line.markers) {
                        for (const toggle of markers.toggle_frames) {
                            if (toggle === keyframe_index) {
                                toggles.push(markers.element);
                            }
                        }
                    }
                }
                return toggles;
            });
        }
    }
    CodeSample.Code.formatters.push(new MarkersFormatter());
})(CodeSample || (CodeSample = {}));
//# sourceMappingURL=markers.js.map