var CodeSample;
(function (CodeSample) {
    class SyntaxHighlightingFormatter {
        constructor(optimise) {
            this.optimise = optimise;
        }
        preprocess_line(line) {
            if (!this.optimise) {
                return;
            }
            // Optimisation: collapse token when it's solely whitespace
            const new_tokens = [];
            line.tokens.forEach((token, index) => {
                const substring = line.content.substring(token.range[0], token.range[1]);
                if (substring.trim() !== '') {
                    new_tokens.push(token);
                    return;
                }
                // Last token, cannot collapse into next
                if (index === line.tokens.length - 1) {
                    // Sole token
                    if (new_tokens.length === 0) {
                        new_tokens.push(token);
                    }
                    else {
                        new_tokens[new_tokens.length - 1].range[1] = token.range[1];
                    }
                }
                else {
                    line.tokens[index + 1].range[0] = token.range[0];
                }
            });
            line.tokens = new_tokens;
        }
        process_set(lines, set) { }
        render(lines, set) {
            lines.forEach((line, index) => {
                const wrapper = $('<span class="collapse">').text(index + 1);
                line.line_number = [wrapper];
            });
            for (const line of lines) {
                const wrapper = $('<div class="collapse">');
                const content = $('<span class="content">').appendTo(wrapper);
                for (const token of line.tokens) {
                    $('<span>')
                        .css('color', token.color)
                        .text(line.content.substring(token.range[0], token.range[1]))
                        .appendTo(content);
                }
                line.source_code = [wrapper];
            }
        }
    }
    CodeSample.Code.formatters.push(new SyntaxHighlightingFormatter(false));
})(CodeSample || (CodeSample = {}));
//# sourceMappingURL=syntax_highlighting.js.map