var Slides;
(function (Slides) {
    Slides.modules = [];
    const main = function () {
        if (navigator.userAgent && navigator.userAgent.indexOf('Chrome') !== -1
            && navigator.userAgent.indexOf('Edge') === -1) {
            $('body').addClass('chrome');
        }
        http_get('content/slides.json')
            .then(text => parse_json(text))
            .then(load_slides)
            .then(parse_slides)
            .then(x => present_slides(x.sections, x.slides))
            .then(() => console.log('data loaded'));
    };
    const load_slides = function (slide_names) {
        return Promise.all(map(slide_names, slide => http_get(`content/${slide}.html`)))
            .then(function (values) {
            return values.join('\n');
        });
    };
    const parse_slides = function (sections_html) {
        let global_index = 0;
        const sections = map($(sections_html).filter('section'), (section, section_index) => {
            const $section = $(section);
            let footnote = $section.data('footnote');
            if (!footnote) {
                footnote = undefined;
            }
            else {
                footnote = footnote.toString();
            }
            const slides = $section.children('article');
            const section_obj = {
                index: section_index,
                footnote: footnote,
                slides: []
            };
            return {
                section: section_obj,
                slide_data: map(slides, (slide_html, slide_index) => {
                    return parse_slide(slide_html, global_index++, section_obj, slide_index);
                })
            };
        });
        const section_objs = sections.map(x => x.section);
        const slide_objs = sections.map(x => x.slide_data).reduce((a, b) => a.concat(b));
        return Promise.all(slide_objs).then(s => {
            s.forEach(slide => slide.section.slides.push(slide));
            return {
                sections: section_objs,
                slides: s
            };
        });
    };
    const parse_slide = function (element, global_index, section_obj, slide_index) {
        const $element = $(element);
        const slide = {
            global_index: global_index,
            section: section_obj,
            slide_index: slide_index,
            keyframes: [],
            keyframe_index: 0,
            element: $element
        };
        const module_promises = [];
        for (const module of Slides.modules) {
            module_promises.push(module($element));
        }
        // Render all results
        const module_results = Promise.all(module_promises)
            .then(arrays => arrays.reduce((a, b) => a.concat(b)))
            .then(results => results.map(result => result.renderer.render(result.element)));
        // Takes the keyframes from all module results, and then
        // compresses them into one linear array of keyframes.
        return module_results.then(results => {
            if (results.length === 0) {
                return slide;
            }
            const min_frame_offset = results.reduce((min, value) => value.frame_offset < min ? value.frame_offset : min, Number.MAX_SAFE_INTEGER);
            results.forEach(x => x.frame_offset -= min_frame_offset);
            let frames = [];
            results.forEach(c => {
                c.keyframes.forEach((kf, i) => {
                    let kfs = frames[i + c.frame_offset];
                    if (!kfs) {
                        kfs = [];
                        frames[i + c.frame_offset] = kfs;
                    }
                    kfs.push(kf);
                });
            });
            slide.keyframes = frames.filter(x => x);
            return slide;
        });
    };
    const present_slides = function (sections, slides) {
        slides.forEach(slide => {
            const meta = $('<div class="slide-meta">');
            $('<div class="slide-position">')
                .text(`${slide.section.index + 1}.${slide.slide_index + 1} (${slide.global_index + 1})`)
                .appendTo(meta);
            if (slide.section.footnote) {
                $('<div class="slide-footnote">').text(slide.section.footnote).appendTo(meta);
            }
            meta.prependTo(slide.element);
        });
        const $main = $('main');
        for (let i = 1; i < slides.length; i++) {
            const slide = slides[i];
            slide.element.css({
                left: slide.section.index * 100 + '%',
                top: (slide.global_index - slide.section.index) * 100 + '%',
            });
        }
        if (slides.length >= 2) {
            slides[1].element.addClass('visible');
        }
        if (slides.length >= 1) {
            slides[0].element.addClass('visible');
        }
        for (const slide of slides) {
            $main.append(slide.element);
        }
        const section_to_slide = function (section_index, subslide_index) {
            section_index = Math.max(0, Math.min(sections.length - 1, section_index));
            const section = sections[section_index];
            subslide_index = Math.max(0, Math.min(section.slides.length - 1, subslide_index));
            return section.slides[subslide_index].global_index;
        };
        let active_slide_index = 0;
        const update_main_position = function () {
            const slide = slides[active_slide_index];
            $main.css({
                left: slide.section.index * -100 + '%',
                top: (slide.global_index - slide.section.index) * -100 + '%'
            });
        };
        const move_next_slide = function () {
            while (slides[active_slide_index].keyframe_index
                !== slides[active_slide_index].keyframes.length) {
                move_next_step(false);
            }
            if (active_slide_index === slides.length - 1) {
                return;
            }
            active_slide_index++;
            if (active_slide_index >= 2) {
                slides[active_slide_index - 2].element.removeClass('visible');
            }
            if (active_slide_index + 1 < slides.length) {
                slides[active_slide_index + 1].element.addClass('visible');
            }
            update_main_position();
        };
        const move_previous_slide = function () {
            while (slides[active_slide_index].keyframe_index !== 0) {
                move_previous_step(false);
            }
            if (active_slide_index === 0) {
                return;
            }
            active_slide_index--;
            if (active_slide_index + 2 < slides.length) {
                slides[active_slide_index + 2].element.removeClass('visible');
            }
            if (active_slide_index >= 1) {
                slides[active_slide_index - 1].element.addClass('visible');
            }
            update_main_position();
        };
        const move_next_step = function (allow_slide_transition) {
            const slide = slides[active_slide_index];
            if (slide.keyframe_index === slide.keyframes.length) {
                if (allow_slide_transition) {
                    move_next_slide();
                }
                return true;
            }
            for (const action of slide.keyframes[slide.keyframe_index]) {
                action.apply();
            }
            slide.keyframe_index++;
            return false;
        };
        const move_previous_step = function (allow_slide_transition) {
            const slide = slides[active_slide_index];
            // Start of slide reached
            if (slide.keyframe_index === 0) {
                if (allow_slide_transition) {
                    move_previous_slide();
                }
                return true;
            }
            slide.keyframe_index--;
            for (const action of slide.keyframes[slide.keyframe_index]) {
                action.undo();
            }
            return false;
        };
        const move_to_step = function (step) {
            const slide = slides[active_slide_index];
            step = Math.max(0, Math.min(slide.keyframes.length, step));
            while (slide.keyframe_index < step) {
                move_next_step(false);
            }
            while (slide.keyframe_index > step) {
                move_previous_step(false);
            }
        };
        const move_first_step = function () {
            while (!move_previous_step(false)) { }
        };
        const move_last_step = function () {
            while (!move_next_step(false)) { }
        };
        const move_to_slide = function (slide_index) {
            slide_index = Math.max(0, Math.min(slides.length - 1, slide_index));
            while (slide_index < active_slide_index) {
                move_previous_slide();
            }
            while (slide_index > active_slide_index) {
                move_next_slide();
            }
        };
        const move_next_section = function () {
            const target = slides[active_slide_index].section.index + 1;
            move_to_slide(section_to_slide(target, 0));
            move_first_step();
        };
        const move_previous_section = function () {
            const target = slides[active_slide_index].section.index - 1;
            move_to_slide(section_to_slide(target, 0));
            move_first_step();
        };
        const jump_to_slide = function () {
            const slide_index = prompt(`Go to slide (1-${slides.length} [e|s])`) || '';
            const match = slide_index.match(/^\s*(\d+)\s*([es])?\s*$/);
            if (!match) {
                return;
            }
            const slide_nr = parseInt(match[1], 10) - 1;
            move_to_slide(slide_nr);
            if (match[2] === 's') {
                move_first_step();
            }
            if (match[2] === 'e') {
                move_last_step();
            }
        };
        const jump_to_section = function () {
            const section_slide_index = prompt(`Go to section (1-${sections.length}[.1-n] [e|s])`) || '';
            const match = section_slide_index.match(/^\s*(\d+)(?:\.(\d+))?\s*([es])?\s*$/);
            if (!match) {
                return;
            }
            const slide_index = section_to_slide(parseInt(match[1], 10) - 1, match[2] ? (parseInt(match[2], 10) - 1) : 0);
            move_to_slide(slide_index);
            if (match[3] === 's') {
                move_first_step();
            }
            if (match[3] === 'e') {
                move_last_step();
            }
        };
        // History
        const update_history = function () {
            const slide = slides[active_slide_index];
            window.history.replaceState(undefined, document.title, `#${slide.section.index + 1},${slide.slide_index + 1},${slide.keyframe_index + 1}`);
        };
        {
            const hash_reg = /^#(\d+),(\d+)(?:,(\d+))?$/g;
            const match = hash_reg.exec(window.location.hash);
            if (match) {
                const section = parseInt(match[1], 10) - 1;
                const subslide = parseInt(match[2], 10) - 1;
                const keyframe = match[3] ? parseInt(match[3], 10) - 1 : 0;
                $main.addClass('no-transition');
                setTimeout(() => $main.removeClass('no-transition'), 1);
                move_to_slide(section_to_slide(section, subslide));
                move_to_step(keyframe);
            }
            update_history();
        }
        let presenter_mode = false;
        $('body').keydown(e => {
            switch (e.keyCode) {
                case 71 /* G */:
                    {
                        jump_to_slide();
                    }
                    break;
                case 74 /* J */:
                    {
                        jump_to_section();
                    }
                    break;
                case 120 /* F9 */:
                    {
                        presenter_mode = !presenter_mode;
                    }
                    break;
                case 68 /* D */:
                case 39 /* RIGHT */:
                    {
                        move_next_step(true);
                    }
                    break;
                case 65 /* A */:
                case 37 /* LEFT */:
                    {
                        move_previous_step(true);
                    }
                    break;
                case 83 /* S */:
                case 40 /* DOWN */:
                    {
                        move_next_slide();
                    }
                    break;
                case 87 /* W */:
                case 38 /* UP */:
                    {
                        move_previous_slide();
                    }
                    break;
                case 81 /* Q */:
                    {
                        move_previous_section();
                    }
                    break;
                case 69 /* E */:
                    {
                        move_next_section();
                    }
                    break;
                case 82 /* R */:
                    {
                        move_first_step();
                    }
                    break;
                case 70 /* F */:
                    {
                        move_last_step();
                    }
                    break;
                default: return;
            }
            update_history();
            e.preventDefault();
            e.stopPropagation();
            return false;
        }).mousedown(e => {
            if (!presenter_mode) {
                return;
            }
            switch (e.button) {
                case 0 /* LEFT */:
                    {
                        move_next_step(true);
                    }
                    break;
                case 2 /* RIGHT */:
                    {
                        move_previous_step(true);
                    }
                    break;
                case 1 /* MIDDLE */:
                    {
                        $('body').css('display', 'none');
                        setTimeout(function () { $('body').css('display', ''); }, 1);
                    }
                    break;
                default: return;
            }
            update_history();
            e.preventDefault();
            e.stopPropagation();
            return false;
        }).contextmenu(e => {
            if (presenter_mode) {
                e.preventDefault();
                e.stopPropagation();
                return false;
            }
        }).bind('mousewheel', function (e) {
            if (e.originalEvent.wheelDelta > 0) {
                move_previous_step(true);
            }
            else {
                move_next_step(true);
            }
            update_history();
            e.preventDefault();
            e.stopPropagation();
            return false;
        });
    };
    $(main);
})(Slides || (Slides = {}));
//# sourceMappingURL=main.js.map